@@ -135,7 +135,7 @@ static void folder_update_linestatus (gpointer key, gpointer value, gpointer dat
 static void free_folder (gpointer value);
 static gboolean get_folder_online (MailStubExchangeFolder *mfld, gboolean background);
 
-static GStaticMutex g_changed_msgs_mutex = G_STATIC_MUTEX_INIT;
+static GStaticRecMutex g_changed_msgs_mutex = G_STATIC_REC_MUTEX_INIT;
 
 static void
 class_init (GObjectClass *object_class)
@@ -1916,7 +1916,7 @@ build_message_from_document (E2kContext *ctx, E2kOperation *op,
 	E2kHTTPStatus status;
 	E2kResult *results;
 	int nresults = 0;
-	GByteArray *message;
+	GString *message;
 	char *headers;
 
 	status = e2k_context_propfind (ctx, op, uri,
@@ -1930,15 +1930,15 @@ build_message_from_document (E2kContext *ctx, E2kOperation *op,
 
 	headers = mail_util_mapi_to_smtp_headers (results[0].props);
 
-	message = g_byte_array_new ();
-	g_byte_array_append (message, headers, strlen (headers));
+	message = g_string_new (headers);
+	g_string_append_len (message, *body, *len);
+
 	g_free (headers);
-	g_byte_array_append (message, *body, *len);
 	g_free (*body);
 
-	*body = (char *)message->data;
 	*len = message->len;
-	g_byte_array_free (message, FALSE);
+	*body = g_string_free (message, FALSE);
+
 	e2k_results_free (results, nresults);
 	return status;
 }
@@ -1949,7 +1949,6 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 				    char **body, int *len)
 {
 	const char *prop = PR_RCVD_REPRESENTING_EMAIL_ADDRESS;
-	GByteArray *message;
 	char *delegator_dn, *delegator_uri;
 	ExchangeAccount *account;
 	E2kGlobalCatalog *gc;
@@ -1997,16 +1996,17 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 	if (delegator_uri) {
 		folder = exchange_account_get_folder (account, delegator_uri);
 		if (folder) {
-			message = g_byte_array_new ();
-			g_byte_array_append (message, *body, *len);
+			GString *message;
+
+			message = g_string_new_len (*body, *len);
 			mail_util_demangle_delegated_meeting (
 				message, entry->display_name,
 				entry->email,
 				e_folder_get_physical_uri (folder));
 			g_free (*body);
-			*body = (char *)message->data;
+			*body = message->str;
 			*len = message->len;
-			g_byte_array_free (message, FALSE);
+			*body = g_string_free (message, FALSE);
 		}
 		g_free (delegator_uri);
 	}
@@ -2373,7 +2373,7 @@ get_folder_info (MailStub *stub, const char *top, guint32 store_flags)
 				    path[toplen + 1] != '\0')
 					continue;
 			}
-return_data:
+
 			type = e_folder_get_type_string (folder);
 			name = e_folder_get_name (folder);
 			uri = e_folder_get_physical_uri (folder);
@@ -2727,12 +2727,12 @@ subscribe_folder (MailStub *stub, const char *folder_name)
 static void
 unsubscribe_folder (MailStub *stub, const char *folder_name)
 {
-	d(printf ("unsubscribe folder : %s\n", folder_name));
 	MailStubExchange *mse = MAIL_STUB_EXCHANGE (stub);
 	ExchangeAccountFolderResult result;
 	EFolder *folder;
 	char *path, *pub_name;
 
+	d(printf ("unsubscribe folder : %s\n", folder_name));
 	path = g_build_filename ("/", folder_name, NULL);
 	folder = exchange_account_get_folder (mse->account, path);
 	if (!folder) {
